<?php

namespace Drupal\gentle_user_reminder\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for the gentle user reminder entity edit forms.
 */
class GentleUserReminderForm extends ContentEntityForm {

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $result = parent::save($form, $form_state);

    $entity = $this->getEntity();

    $message_arguments = ['%label' => $entity->toLink()->toString()];
    $logger_arguments = [
      '%label' => $entity->label(),
      'link' => $entity->toLink($this->t('View'))->toString(),
    ];

    switch ($result) {
      case SAVED_NEW:
        $this->messenger()->addStatus($this->t('New gentle user reminder %label has been created.', $message_arguments));
        $this->logger('gentle_user_reminder')->notice('Created new gentle user reminder %label', $logger_arguments);
        break;

      case SAVED_UPDATED:
        $this->messenger()->addStatus($this->t('The gentle user reminder %label has been updated.', $message_arguments));
        $this->logger('gentle_user_reminder')->notice('Updated gentle user reminder %label.', $logger_arguments);
        break;
    }

    $form_state->setRedirect('entity.gentle_user_reminder.canonical', ['gentle_user_reminder' => $entity->id()]);

    return $result;
  }

}
